import { initAllComponents } from '..'

export const debugReplacements = {
  B: `<dl>
    <dt>South America</dt>
    <dd>3 extant species of tapir</dd>
    <dt>Asia</dt>
    <dd>1 extant species of tapir</dd>
  </dl>`,
  Back: `<ul>
    <li>First do no harm</li>
    <li>Second:</li>
    <ul>
      <li>heat the wok until the edges are smoking</li>
      <li>add dried chillies and Sichuan pepper</li>
    </ul>
  </ul>`,
  /** Set Beispiele to no content to test that the entry gets hidden */
  Beispiele: '',
  Subdeck: `<span>
    10—Paradigmenwechseln? Partei, Staat und Gesellschaft unter Jiang Zemin
    und Hu Jintao
  </span>`
}

/**
 * Replaces anki tags with test data in the given element.
 *
 * This function only has an effect in environments other than production, e.g.
 * testing of the template in the browser before changes are tested in Anki.
 * During production, these tags are rendered by Anki itself and this function
 * has no effect.
 */
export async function renderAnkiTagsInElement (
  element: Element
): Promise<void> {
  const children = element.childNodes
  for (let i = 0; i < children.length; ++i) {
    const child = children[i]
    if (child.nodeType === Node.ELEMENT_NODE) {
      const element = child as Element
      if (element.tagName !== 'SCRIPT') {
        await renderAnkiTagsInElement(child as Element)
      }
    } else if (child.nodeType === Node.TEXT_NODE) {
      const replacement = await renderTextNode(null, child.textContent ?? '', 0)
      if (replacement !== null) {
        child.replaceWith(...replacement)
      }
    }
  }
}

/**
 * Looks for anki tags in the text content starting with fromIdx and renders
 * them. The rendered contents are appended to the given array of already added
 * elements (or a new list is started if given null). Returns null only if the
 * previously rendered list was already null and no Anki tags were found and
 * thus nothing should be changed about the node with the given text. An empty
 * array would mean that the text consisted only of anki tags and all resulted
 * in empty content, so that the node with the given text should be deleted.
 */
async function renderTextNode (
  rendered: Node[] | null,
  text: string,
  fromIdx: number
): Promise<Node[] | null> {
  const tagStart = text.indexOf('{{', fromIdx)
  if (tagStart >= 0) {
    // tag found
    const tagContentStart = tagStart + '{{'.length
    const tagEnd = text.indexOf('}}', tagContentStart)
    if (tagEnd < 0) {
      throw new Error('Unclosed Anki tag')
    }

    const tagContent = text.substring(tagContentStart, tagEnd)
    const tagNameEndExcl = !tagContent.includes(' ')
      ? tagContent.length
      : tagContent.indexOf(' ')
    const tagName = tagContent.substring(0, tagNameEndExcl)
    // ignore the tag arguments for now
    rendered ??= []

    if (tagStart > fromIdx) {
      // append text before the anki directive if non-empty
      rendered.push(document.createTextNode(text.substring(fromIdx, tagStart)))
    }

    // then the rendered content for the anki directive
    const tagResult = await renderTag(tagName)
    if (tagResult !== null) {
      rendered.push(tagResult)
    }

    // and then render the rest, which could contain more nodes
    // or just text (or nothing)
    rendered = await renderTextNode(rendered, text, tagEnd + '}}'.length)
  } else if (fromIdx < text.length) {
    // no more tags found, append trailing text to already rendered content
    rendered ??= []
    rendered.push(document.createTextNode(text.substring(fromIdx)))
  } // else no tags found and no preceding text

  return rendered
}

async function renderTag (name: string): Promise<Node | null> {
  if (name === 'FrontSide') {
    const loading = document.createElement('div')
    loading.innerText = 'Loading front side...'
    await replaceFront(loading)
    return loading
  } else if (name.startsWith('text:')) {
    return renderTagOther(name.substring('text:'.length))
  } else {
    return renderTagOther(name)
  }
}

function renderTagOther (name: string): Node | null {
  if (name in debugReplacements) {
    const html = debugReplacements[name] as string
    if (html.length > 0) {
      const template = document.createElement('div')
      template.innerHTML = html
      if (template.children.length !== 1) {
        throw new Error(`Expected just one child in ${html}`)
      }
      return template.children[0]
    } else {
      // no debug content defined, do not add any children
      return null
    }
  } else {
    // for now all other tags render to their name
    return document.createTextNode(name)
  }
}

async function replaceFront (loading: Element): Promise<void> {
  if (loading.ownerDocument.location.pathname.endsWith('front.html')) {
    // avoid loops
    return
  }

  const frontUrl = getCorrespondingFrontUrl(window.location)
  const res = await fetch(frontUrl)
  const frontHtml = await res.text()
  loading.innerHTML = frontHtml
  await renderAnkiTagsInElement(loading)
  await initAllComponents()
  /* const front = document.createElement('div')
  front.classList.add('front')
  front.innerHTML = frontHtml
  await renderAnkiTagsInElement(front)
  loading.replaceWith(...Array.from(front.childNodes)) */
}

function getCorrespondingFrontUrl (backLocation: Location): URL {
  const pathComponents = backLocation.pathname.split('/')
  pathComponents.pop()
  pathComponents.push('front.html')
  const frontUrl = new URL(backLocation.href)
  frontUrl.pathname = pathComponents.join('/')
  return frontUrl
}
