import type { HanziData } from '../../../build/hanzi-data/hanzi-data'
import { lookup } from './lut'
import { readFile } from 'fs/promises'

export interface GetHanziDataOpts {
  char: string
  kind?: GetHanziDataKind
}

export const enum GetHanziDataKind {
  ANY = 'any',
  DEFAULT = 'default',
  TRADITIONAL = 'traditional'
}

export async function getHanziProp (
  { prop, ...opts }: GetHanziDataOpts & { prop: string }
): Promise<string | string[] | undefined> {
  const data = await getHanziData(opts)
  if (data === undefined) {
    return undefined
  }
  return data[prop] ?? lookup(data, prop)
}

const cached: Record<string, HanziData | undefined> = {}

export async function getHanziData (
  { char, kind }: GetHanziDataOpts
): Promise<HanziData | undefined> {
  if (char.length !== 1) {
    throw new Error(`Can only get data for single chars, got: ${char}`)
  }

  const url = process.env.NODE_ENV === 'production'
    ? `${char}.json` // actual Anki keeps only the last path component
    : `/generated/hanzi-data/${char}.json`
  let data: HanziData | undefined
  try {
    if (typeof document === 'undefined') {
      // read directly from disk when running in test
      const relative = url.substring(1)
      const json = await readFile(relative, { encoding: 'utf-8' })
      data = JSON.parse(json)
    } else {
      // in production and parcel use fetch
      if (char in cached) {
        data = cached[char]
        if (data !== undefined) {
          data = { ...data }
        }
      } else {
        const dataRes = await fetch(url)
        if (dataRes.status !== 200) {
          data = undefined
        } else {
          data = await dataRes.json()
        }
        cached[char] = data === undefined ? undefined : { ...data }
      }
    }
  } catch (err) {
    console.error(err)
    data = undefined
  }
  if (data !== undefined && kind === GetHanziDataKind.TRADITIONAL) {
    for (const overridden in data.trad) {
      data[overridden] = data.trad[overridden]
    }
  }
  if (char === '聽') {
    console.error('data returning: ', data?.radical)
  }
  return data
}
